@@ -217,6 +217,8 @@ gst_ffmpegdemux_base_init (GstFFMpegDemuxClass * klass)
   gst_element_class_add_pad_template (element_class, audiosrctempl);
   gst_element_class_add_pad_template (element_class, sinktempl);
 
+  gst_caps_unref (sinkcaps);
+
   klass->in_plugin = in_plugin;
   klass->videosrctempl = videosrctempl;
   klass->audiosrctempl = audiosrctempl;
@@ -1392,6 +1394,7 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
   gint outsize;
   gboolean rawvideo;
   GstFlowReturn stream_last_flow;
+  gint64 pts;
 
   /* open file if we didn't so already */
   if (!demux->opened)
@@ -1419,7 +1422,21 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
 
   /* do timestamps, we do this first so that we can know when we
    * stepped over the segment stop position. */
-  timestamp = gst_ffmpeg_time_ff_to_gst (pkt.pts, avstream->time_base);
+  pts = pkt.pts;
+  if (G_UNLIKELY (pts < 0)) {
+    /* some streams have pts such this:
+     * 0
+     * -2
+     * -1
+     * 1
+     *
+     * we reset pts to 0 since for us timestamp are unsigned
+     */
+    GST_WARNING_OBJECT (demux,
+        "negative pts detected: %" G_GINT64_FORMAT " resetting to 0", pts);
+    pts = 0;
+  }
+  timestamp = gst_ffmpeg_time_ff_to_gst (pts, avstream->time_base);
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
     stream->last_ts = timestamp;
   }
@@ -2028,8 +2045,9 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
         !strcmp (in_plugin->name, "swf") ||
         !strcmp (in_plugin->name, "voc") ||
         !strcmp (in_plugin->name, "pva") ||
-        !strcmp (in_plugin->name, "gif") || !strcmp (in_plugin->name, "vc1test")
-        )
+        !strcmp (in_plugin->name, "gif") ||
+        !strcmp (in_plugin->name, "vc1test") ||
+        !strcmp (in_plugin->name, "ivf"))
       register_typefind_func = FALSE;
 
     /* Set the rank of demuxers known to work to MARGINAL.
@@ -2065,7 +2083,8 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
         !strcmp (in_plugin->name, "4xm") ||
         !strcmp (in_plugin->name, "yuv4mpegpipe") ||
         !strcmp (in_plugin->name, "pva") ||
-        !strcmp (in_plugin->name, "mpc") || !strcmp (in_plugin->name, "mpc8"))
+        !strcmp (in_plugin->name, "mpc") ||
+        !strcmp (in_plugin->name, "mpc8") || !strcmp (in_plugin->name, "ivf"))
       rank = GST_RANK_MARGINAL;
     else {
       GST_DEBUG ("ignoring %s", in_plugin->name);
